@@ -16,7 +16,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_REGISTERFORMEMPTY
 );
 
-our $VERSION = '2.17.0';
+our $VERSION = '2.18.2';
 
 extends qw(
   Lemonldap::NG::Portal::Main::Plugin
@@ -214,7 +214,7 @@ sub parse {
             $Lemonldap::NG::Common::Safelib::functions );
 
         foreach ( keys %{ $json->{rules} } ) {
-            $cpt->reval("BEGIN { 'warnings'->unimport; } $json->{rules}->{$_}");
+            $cpt->reval($json->{rules}->{$_});
             my $err = join(
                 '',
                 grep(
